/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.manager.ReadingRunnableTest.java
 * Created on: Dec 6, 2016
 */
package fr.bigeon.gclc.manager;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.fail;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;

import org.junit.Before;
import org.junit.Test;

/** <p>
 * TODO
 *
 * @author Emmanuel Bigeon */
public class ReadingRunnableTest {

    /**
     */
    @Before
    public void setUp() {}

    /** Test method for
     * {@link fr.bigeon.gclc.manager.ReadingRunnable#getMessage()}. */
    @Test
    public final void testGetMessage() {
        BufferedReader reader = null;
        ReadingRunnable runnable = new ReadingRunnable(reader);
        runnable.setRunning(false);

        try {
            runnable.getMessage();
            fail("reading from closed runnable");
        } catch (IOException e) {
            assertNotNull(e);
        }

    }

    /** Test method for
     * {@link fr.bigeon.gclc.manager.ReadingRunnable#hasMessage()}. */
    @Test
    public final void testHasMessage() {

        BufferedReader reader = null;
        ReadingRunnable runnable = new ReadingRunnable(reader);
        runnable.setRunning(false);

        try {
            runnable.getMessage();
            fail("reading from closed runnable");
        } catch (IOException e) {
            assertNotNull(e);
        }
    }

    /** Test method for
     * {@link fr.bigeon.gclc.manager.ReadingRunnable#getWaitForDelivery(java.lang.String)}.
     * 
     * @throws InterruptedException */
    @Test
    public final void testGetWaitForDelivery() throws InterruptedException {
        try (PipedOutputStream out = new PipedOutputStream();
             InputStream piped = new PipedInputStream(out);
             BufferedReader reader = new BufferedReader(
                     new InputStreamReader(piped, "UTF-8"))) {
            final ReadingRunnable runnable = new ReadingRunnable(reader);
            Thread th0 = new Thread(runnable, "read");
            th0.start();
            Thread th = runnable.getWaitForDelivery("");

            final Object start = new Object();

            Thread th2 = new Thread(new Runnable() {

                @Override
                public void run() {
                    synchronized (start) {
                        start.notify();
                    }
                    try {
                        runnable.getMessage();
                    } catch (IOException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                }
            }, "get");
            synchronized (start) {
                th2.start();
                start.wait();
            }
            runnable.interrupt();
            try {
                th.join();
            } catch (InterruptedException e) {
                assertNull(e);
            }
            runnable.setRunning(false);
            out.close();
        } catch (IOException e1) {
            assertNull(e1);
        }
    }

}
